﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using EffectiveUI.MSDN.Web.Models;
using System.Drawing;
using System.IO;

namespace EffectiveUI.MSDN.Web.Controllers
{
	public class HomeController : Controller
	{
		/// <summary>
		/// Main page for the site
		/// </summary>
		/// <returns></returns>
		public ActionResult Index()
		{
			ViewBag.BodyID = "1";

			return SelectView("Index", null);
		}

		/// <summary>
		/// Page that displays a Google Map and executes filtering of the map based on a bounding box
		/// </summary>
		/// <param name="maxLat">Maximum latitude of the bounding box</param>
		/// <param name="maxLon">Maximum longitude of the bounding box</param>
		/// <param name="minLat">Minimum latitude of the bounding box</param>
		/// <param name="minLon">Minimum longitude of the bounding box</param>
		/// <param name="outputType">Requested output type ("html" or "json")</param>
		/// <returns></returns>
		public ActionResult Map(float maxLat = 90f, float maxLon = 180f, float minLat = -90f, float minLon = -180f, string outputType = "html")
		{
			ViewBag.BodyID = "2";
			MapModel model = new MapModel();
			model.FilterItems(new LatLong(maxLat, maxLon), new LatLong(minLat, minLon));

			return SelectView("Map", model, outputType);
		}

		/// <summary>
		/// Work Order Details page
		/// </summary>
		/// <param name="id">Work order id to retrieve</param>
		/// <returns></returns>
		public ActionResult Details(string id)
		{
			ViewBag.BodyID = "2";
			MapModel model = new MapModel();
			MapItem item = model.GetItemById(id);
			string path = Url.Content("~/Content/uploaded-images/");

			item.Images.Clear();
			string[] images = Directory.GetFiles(Server.MapPath("~/Content/uploaded-images/"), item.ID + "-*.*");
			foreach (string image in images)
			{
				item.Images.Add(new WorkOrderImage(item.ID, Path.GetFileName(image).Replace(item.ID + "-", "")));
			}

			return SelectView("Details", item);
		}

		/// <summary>
		/// Image upload page for graceful degradation if the user does not have the native mobile application shell
		/// </summary>
		/// <param name="id">The work order id to attach the new image to</param>
		/// <returns></returns>
		public ActionResult Image(string id)
		{
			ViewBag.BodyID = "2";
			MapModel mapModel = new MapModel();
			MapItem item = mapModel.GetItemById(id);
			ImageUploadModel model = new ImageUploadModel() { ID = item.ID, Issue = item.Issue };

			return SelectView("Image", model);
		}

		/// <summary>
		/// Endpoint to post the uploaded picture to
		/// </summary>
		/// <param name="id">Work order id to attach the picture to</param>
		/// <param name="photo">The uploaded photo object</param>
		/// <returns></returns>
		[HttpPost]
		public ActionResult UploadPicture(string id, HttpPostedFileBase photo)
		{
			if (photo != null)
			{
				Int32 length = photo.ContentLength;

				try
				{
					if (length != 0)
					{
						if (photo.ContentType.StartsWith("image"))
						{
							Bitmap bmp = new Bitmap(photo.InputStream);
							bmp.Save(Server.MapPath("~/Content/uploaded-images/") + id + "-" + Path.GetFileNameWithoutExtension(photo.FileName) + ".png", System.Drawing.Imaging.ImageFormat.Png);
						}
					}
					else
					{
						//TODO: Return an error message to the user
					}
				}
				catch
				{
					//TODO: Return an error message to the user
				}
			}
			return this.RedirectToAction("Details", "Home", new { id = id });
		}

		/// <summary>
		/// Page to view an uploaded image
		/// </summary>
		/// <param name="id">Image id to view</param>
		/// <returns></returns>
		public ActionResult ViewImage(string id)
		{
			ViewBag.BodyID = "2";
			WorkOrderImage image = new WorkOrderImage();
			image.ID = id.Substring(0, id.IndexOf('-'));
			image.FileName = id.Substring(id.IndexOf('-') + 1);

			return SelectView("ViewImage", image);
		}

		/// <summary>
		/// Helper function to select the view to return based on the selected output type and the browser type
		/// </summary>
		/// <param name="viewName">Name of the view to return</param>
		/// <param name="model">Model to send to the selected view</param>
		/// <param name="outputType">Requested output type ("html" or "json")</param>
		/// <returns>ActionResult based on the named view, output type, and browser type</returns>
		private ActionResult SelectView(string viewName, object model, string outputType = "html")
		{
			if (outputType.ToLower() == "json")
			{
				return Json(model, JsonRequestBehavior.AllowGet);
			}
			else
			{
#if MOBILE
				return View(viewName + "Mobile", model);
#else
				if (Request.Browser.IsMobileDevice)
				{
					return View(viewName + "Mobile", model);
				}
				else
				{
					return View(viewName, model);
				}
#endif
			}
		}
	}
}
