﻿//MapModel.cs
//Copyright © EffectiveUI 2011

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using EffectiveUI.MSDN.Web.Utility;
using System.IO;

namespace EffectiveUI.MSDN.Web.Models
{
	/// <summary>
	/// Represents a latitude and longitude coordinate
	/// </summary>
	public class LatLong
	{
		public LatLong() { }
		public LatLong(float lat, float lon)
		{
			Latitude = lat;
			Longitude = lon;
		}

		public float Latitude { get; set; }
		public float Longitude { get; set; }
	}

	public class WorkOrderImage
	{
		public WorkOrderImage()
		{
			ID = "";
			FileName = "";
		}

		public WorkOrderImage(string id, string fileName)
		{
			ID = id;
			FileName = fileName;
		}

		public string ID { get; set; }
		public string FileName { get; set; }
	}

	/// <summary>
	/// Represents a map item
	/// </summary>
	public class MapItem
	{
		public MapItem()
		{
			ID = "";
			Issue = "";
			Task = "-";
			AssetType = "-";
			Assignment = "-";
			Status = "";
			Location = new List<LatLong>();
			Images = new List<WorkOrderImage>();
		}

		public string ID { get; set; }
		public string Issue { get; set; }
		public string Task { get; set; }
		public string AssetType { get; set; }
		public string Assignment { get; set; }
		public string Status { get; set; }
		public List<LatLong> Location { get; set; }
		public List<WorkOrderImage> Images { get; set; }
	}

	/// <summary>
	/// Main Map Model
	/// </summary>
	public class MapModel
	{
		static List<MapItem> _allItems = LoadData();

		public MapModel()
		{
			Items = MapModel._allItems;
		}

		public List<MapItem> Items { get; set; }

		/// <summary>
		/// Filters the Items list by the rectangle formed by the specified bounding coordinates
		/// </summary>
		/// <param name="ne">The Northeast corner of the bounding rectangle</param>
		/// <param name="sw">The Southwest corner of the bounding rectangle</param>
		public void FilterItems(LatLong ne, LatLong sw)
		{
			var items = (from i in _allItems.AsParallel()
						 where i.Location.Max(m => m.Latitude) <= ne.Latitude &&
							i.Location.Min(m => m.Latitude) >= sw.Latitude &&
						 i.Location.Max(m => m.Longitude) <= ne.Longitude &&
						 i.Location.Min(m => m.Longitude) >= sw.Longitude
						 select i);
			Items = items.ToList();
		}

		/// <summary>
		/// Gets a specific MapItem by id
		/// </summary>
		/// <param name="id">ID of the item to find</param>
		/// <returns>Selected MapItem</returns>
		public MapItem GetItemById(string id)
		{
			var item = (from i in _allItems.AsParallel()
						where i.ID == id
						select i);
			return item.FirstOrDefault();
		}

		/// <summary>
		/// Loads the initial data into the model
		/// </summary>
		/// <returns></returns>
		private static List<MapItem> LoadData()
		{
			string filename = "MockWorkOrderData.xml";
			if (!File.Exists(EffectiveUI.MSDN.Web.Utility.Utility.GetAssemblyPath() + Path.DirectorySeparatorChar + filename))
			{
				GenerateRandomData(filename);
			}
			SampleDataLoader loader = new SampleDataLoader();

			List<MapItem> results = loader.LoadSampleData<List<MapItem>>(filename);
			return results;
		}

		/// <summary>
		/// Create mock data for demonstration purposes roughly following wireframes version 102
		/// </summary>
		private static void GenerateRandomData(string filename)
		{
			Random r = new Random();
			LatLong baseLatLong = new LatLong(39.75517f, -104.99099089999998f);

			List<MapItem> items = new List<MapItem>();

			items.Add(new MapItem() { ID = "1234", Issue = "Routine Maintenance", Task = "Inspect", AssetType = "Stop Sign", Assignment = "Team A", Status = "In Progress" });
			items.Last().Location = new List<LatLong>() { baseLatLong };
			items.Add(new MapItem() { ID = "2345", Issue = "Routine Maintenance", Status = "In Progress" });
			items.Add(new MapItem() { ID = "3456", Issue = "Vandalism", Status = "In Progress" });
			items.Add(new MapItem() { ID = "4567", Issue = "Missing Sign", Task = "Inspect", AssetType = "Street Sign", Assignment = "Bill Smith", Status = "Submitted" });
			items.Add(new MapItem() { ID = "5678", Issue = "Routine Maintenance", Task = "Inspect", AssetType = "Street Sign", Assignment = "Team A", Status = "In Progress" });
			items.Add(new MapItem() { ID = "6789", Issue = "Needs Paint", Task = "Repair", AssetType = "Stop Sign", Assignment = "Team A", Status = "In Progress" });
			items.Add(new MapItem() { ID = "7890", Issue = "Needs Paint", Task = "Repair", AssetType = "Stop Sign", Assignment = "Team A", Status = "In Progress" });
			items.Add(new MapItem() { ID = "8901", Issue = "Low Reflectivity", Status = "Submitted" });
			items.Add(new MapItem() { ID = "9012", Issue = "Low Reflectivity", Status = "Submitted" });
			items.Add(new MapItem() { ID = "9123", Issue = "Vandalism", Status = "Submitted" });
			items.Add(new MapItem() { ID = "9234", Issue = "Rust", Status = "Deferred" });
			items.Add(new MapItem() { ID = "9345", Issue = "Low Reflectivity", Status = "In Progress" });
			items.Add(new MapItem() { ID = "9456", Issue = "Vandalism", Status = "Submitted" });
			items.Add(new MapItem() { ID = "9567", Issue = "Rust", Status = "Deferred" });
			items.Add(new MapItem() { ID = "9678", Issue = "Low Reflectivity", Status = "In Progress" });
			for (int i = 1; i < items.Count; i++)
			{
				double randSignLat = r.NextDouble();
				double randLat = r.NextDouble() / 100;
				double randSignLon = r.NextDouble();
				double randLon = r.NextDouble() / 100;

				if (randSignLat < 0.5) { randLat = -randLat; }
				if (randSignLon < 0.5) { randLon = -randLon; }

				items[i].Location = new List<LatLong>() { new LatLong(baseLatLong.Latitude + (float)randLat, baseLatLong.Longitude + (float)randLon) };
			}

			SampleDataLoader loader = new SampleDataLoader();
			loader.SaveSampleData(filename, items);
		}
	}
}